<?php

namespace xenarabia\MYMMSMSG\XF\Admin\Controller;
use XF\Mvc\FormAction;
use XF\Mvc\ParameterBag;
use XF\Mvc\Reply\View;
use XF\Admin\Controller\AbstractController;

use xenarabia\MYMMSMSG\XF\Entity\CatMyMms;
use xenarabia\MYMMSMSG\XF\Entity\MmsMsgs As MmsMsgsEntity;

class MmsMsgs extends AbstractController
{
	protected $uploadImageName;

	public function actionIndex(ParameterBag $params)
	{
		$page = $this->filterPage($params->page);
		$perPage = 20;
        $mmsmsgs = $this->finder('xenarabia\MYMMSMSG\XF:MmsMsgs')->limitByPage($page, $perPage);
        
		$viewParams = [
			'mmsmsgs'	=> $mmsmsgs->fetch(),
            'total' => $mmsmsgs->total(),
            'page' => $page,
            'perPage' => $perPage,

		];
		return $this->view('xenarabia\MYMMSMSG\XF:mms\view', 'mms_view_list', $viewParams);
	}

	
	public function actionViewByCat(ParameterBag $params)
	{
		$catId = $this->filter('cat_id', 'int');
		$page = $this->filterPage($params->page);
		$perPage = 20;
        $mmsmsgs = $this->finder('xenarabia\MYMMSMSG\XF:MmsMsgs')
		->where('mms_cat_id', $catId)
		->with('CatMyMms')->limitByPage($page, $perPage);
		$catName = $this->finder('xenarabia\MYMMSMSG\XF:CatMyMms')->where('cat_id', $catId)->fetchOne();
		$viewParams = [
			'mmsmsgs'	=> $mmsmsgs->fetch(),
            'total' => $mmsmsgs->total(),
            'page' => $page,
            'perPage' => $perPage,
			'catId' => $catId, 
			'catName' => $catName->cat_title
			
		];
		// \XF::dump($page);
		return $this->view('xenarabia\MYMMSMSG\XF:mmsmsg\view', 'mmsmsgs_by_cat_list', $viewParams);
	}

	public function actionViewImg(ParameterBag $params)
	{
		$showimg = $this->finder('xenarabia\MYMMSMSG\XF:MmsMsgs')->where('mms_id', $params->mms_id)->fetchOne();
		return $this->view('xenarabia\MYMMSMSG\XF:mmsmsg\view', 'mmsmsgs_show_img', ['showimg' => $showimg]);
	}

	public function actionAdd()
	{
		
		$catmms = $this->finder('xenarabia\MYMMSMSG\XF:CatMyMms')->fetch();
		return $this->view('xenarabia\MYMMSMSG\XF:MYMMSMSG\add', 'mms_addedit', ['catmms' => $catmms]);
	}

	
	public function actionSave(ParameterBag $params)
	{
		$this->assertPostOnly();
		if ($params->mms_id) {
			$mms = $this->assertMmsExists($params->mms_id);
			$form = $this->mmsEditSave($mms);
		} else {
			$mms = $this->em()->create('xenarabia\MYMMSMSG\XF:MmsMsgs');
			$form = $this->pageSaveProcess($mms);
		}

		$cat_id = $this->filter('mms_cat_id', 'int');

		if (get_class($form) === 'XF\Mvc\Reply\Error') {
			return $form;
		} else {
			$form->run();
		}

		return $this->redirect($this->buildLink('mymmsmsgs/view-by-cat&cat_id='.$cat_id), \XF::phrase('your_changes_have_been_saved'));
	}

	/** @var xenarabia\MYMMSMSG\XF\Entity\MmsMsgs $mmsMsgsEntity */
	protected function mmsEditSave(MmsMsgsEntity $mmsMsgsEntity)
	{
		$entityInput = $this->filter([
									'mms_id' => 'int',
									'mms_cat_id' => 'int',
									'mms_img_name' => 'str',
									'mms_act' => 'int'
									]);

		$form = $this->formAction();
		$form->basicEntitySave($mmsMsgsEntity, $entityInput);

		return $form;
	}

	/** @var xenarabia\MYMMSMSG\XF\Entity\MmsMsgs $mmsMsgsEntity */
	protected function pageSaveProcess(MmsMsgsEntity $mmsMsgsEntity)
	{

		$form = $this->formAction();
		$mms_cat_id = $this->filter('mms_cat_id', 'int');
		$form->validate(function (FormAction $form) use ($mms_cat_id) {
			if ($mms_cat_id === '' || empty($mms_cat_id)) {
				$form->logError(\XF::phrase('mms_you_don_not_choose_mms_cat'), 'mms_cat_id');
			}
		});
		
		$entityInput = $this->filter([
									'mms_cat_id' => 'int',
									'mms_act' => 'int'
									]);
		if ($this->filter('mms_img_link', 'str')) {
			$entityInput['mms_img_name'] = $this->filter('mms_img_link', 'str');
		}else{
			$upload = $this->request->getFile('mms_img_upload_input', false);
			if ($upload) {
				$this->uploadImageName = \XF::$time . "-MmsImages." . $upload->getExtension();
				$dataFile = 'data://my_mms/' . $this->uploadImageName;
				\XF\Util\File::copyFileToAbstractedPath($upload->getTempFile(), $dataFile);
			} else {
				return $this->error(\XF::phrase('mms_you_don_not_choose_image_to_upload_it'));
			}
			$entityInput['mms_img_name'] = $this->uploadImageName;
		}
		

		$form->basicEntitySave($mmsMsgsEntity, $entityInput);

		return $form;
	}

	
	public function actionMultiMmsUpload()
	{
		$mms_dir = 'data/my_mms';
		$mms_folders = [];
		foreach (scandir($mms_dir) as $fol) {
			if(is_dir($mms_dir.'/'.$fol) && $fol != "." && $fol != ".." && ( count(scandir($mms_dir.'/'.$fol)) - 2 > 0))
			{
				$mms_folders[] =  $fol;
			}
		}
		$catmms = $this->finder('xenarabia\MYMMSMSG\XF:CatMyMms')->fetch();

		$viewParams = [
			'catmms' => $catmms, 
			'mms_folders' => $mms_folders,

		];
		
		return $this->view('xenarabia\MYMMSMSG\XF:MYMMSMSG\Upload_Multi_Mms', 'mms_multi_mms', $viewParams);
	}
	
	public function actionMultiMmsUploadSave(ParameterBag $params)
	{
		$mms_cat_id = $this->filter('mms_cat_id', 'int');
		$selected_folder = $this->filter('mms_folder', 'str');
		$selected_folder_dir = 'data/my_mms/' . $selected_folder;

		$form = $this->formAction();
		$form->validate(function (FormAction $form) use ($mms_cat_id, $selected_folder) {
			if ($mms_cat_id === '' || empty($mms_cat_id)) {
				$form->logError(\XF::phrase('mms_you_don_not_choose_mms_cat'), 'mms_cat_id');
			}

			if ($selected_folder === '' || empty($selected_folder)) {
				$form->logError(\XF::phrase('mms_you_don_not_choose_mms_msgs_folder'), 'mms_folder');
			}
		});

		foreach(glob($selected_folder_dir.'/*.*') as $file) 
		{
			$exclude = array("gif","jpeg","jpg","jpe","png");
			$extension = pathinfo($file, PATHINFO_EXTENSION);
			if(in_array($extension, $exclude))
			{
				$mmsMsgsEntity = $this->em()->create('xenarabia\MYMMSMSG\XF:MmsMsgs');
				$entityInput['mms_cat_id'] = $mms_cat_id;
				$entityInput['mms_img_name'] = substr(substr($file, 11), 1);
				$entityInput['mms_act'] = 1;
				$form->basicEntitySave($mmsMsgsEntity, $entityInput);
			}
			
		}
		
		if (get_class($form) === 'XF\Mvc\Reply\Error') {
			return $form;
		}else {
			$form->run();
			return $this->redirect($this->buildLink('mymmsmsgs/view-by-cat&cat_id=' . $mms_cat_id), \XF::phrase('your_changes_have_been_saved'));
		}

	}

	public function actionDelete(ParameterBag $params)
	{
		$mms = $this->assertMmsExists($params->mms_id, 'CatMyMms');
		$cat_id = $mms->mms_cat_id;

		/** @var \XF\ControllerPlugin\Delete $plugin */
		$plugin = $this->plugin('XF:Delete');

		// Delete image from folder
		$imgPath = \XF::getRootDirectory() . "/data/my_mms/" . $mms->mms_img_name;
		file_exists($imgPath) ? @unlink($imgPath) : null;

		return $plugin->actionDelete(
			$mms,
			$this->buildLink('xa-mms-msgs/delete', $mms),
			$this->buildLink('mymmsmsgs/edit', $mms),
			$this->buildLink('mymmsmsgs/view-by-cat', null, ['cat_id' => $cat_id]),
			$mms->mms_img_name
		);
	}

	public function actionChangeStatus(ParameterBag $params)
    {
        $status = $this->filter('status', 'int');
		$cat_id = $this->filter('cat_id', 'int');

		$updateMsg = \XF::db()->update('xa_my_mms',['mms_act' => $status], 'mms_id = ?', [$params->mms_id]);

		// \XF::dump($params->mms_id);
		if ($updateMsg) {
			return $this->redirect($this->buildLink('mymmsmsgs/view-by-cat&cat_id=' . $cat_id), \XF::phrase('your_changes_have_been_saved'));
		}else{
			return $this->redirect($this->buildLink('mymmsmsgs/view-by-cat&cat_id=' . $cat_id), \XF::phrase('something_went_wrong_please_try_again'));
		}

    }
	
	public function actionMoveMultiMms(ParameterBag $params)
	{
		$mms = $this->request->get('mms_id');
		$cat_id = $this->filter('cat_id', 'int');
		$mms_cat_id = $this->filter('mms_cat_id', 'int');
		foreach ($mms as $img_id) {
			\XF::db()->update('xa_my_mms',['mms_cat_id' => $mms_cat_id], 'mms_id = ? AND mms_cat_id = ?', [$img_id, $cat_id]);
		}
		// \XF::dump($mms_cat_id);
		return $this->redirect($this->buildLink('mymmsmsgs/view-by-cat&cat_id=' . $mms_cat_id), \XF::phrase('your_changes_have_been_saved'));
	}

	public function actionDeleteMultiMms(ParameterBag $params)
	{
		$mms = $this->request->get('mms_del');
		$cat_id = $this->filter('cat_id', 'int');		
		$page = $this->filter('page', 'int');		
		$move_mms = $this->filter('move_mms', 'int');		
		if ($move_mms) {
			$viewParams = [
				'mms' => $mms,
				'catid' => $cat_id,
				'cats' => $this->finder('xenarabia\MYMMSMSG\XF:CatMyMms')->fetch()
			];
			
			return $this->view('xenarabia\MYMMSMSG\XF:MYMMSMSG\Upload_Multi_Mms', 'mms_move_multi_mms', $viewParams);
		}
		// \XF::dump($move_mms);
		if (!$mms) {
			if ($page !== 1) {
				$page_params = $cat_id . "&page=" .$page;
			}else{
				$page_params = $cat_id;
			}
			return $this->redirect($this->buildLink('mymmsmsgs/view-by-cat&cat_id=' . $page_params));
		}
		foreach ($mms as $mmsId) {
			$mms_msgs[] = $this->assertMmsExists($mmsId);
		}

		$viewParams = [
			'mms_msgs' => $mms_msgs,
			'page' => $page,
			'cat_id' => $cat_id
		];
		
		return $this->view('xenarabia\MYMMSMSG\XF:MYMMSMSG\DeleteMultiMms', 'mms_confirm_delete_multi_msgs', $viewParams);
	}

	public function actionSaveMultiDelete(ParameterBag $params)
	{
		$mms_ids = $this->request->get('mms_id');
		$cat_id = $this->filter('cat_id', 'int');		
		$page = $this->filter('page', 'int');
		// \XF::dump($mms_ids);
		foreach ($mms_ids as $mms_id) {
			$imgNmae =$this->assertMmsExists($mms_id);
			$imgPath = \XF::getRootDirectory() . "/data/my_mms/" . $imgNmae->mms_img_name;
			file_exists($imgPath) ? @unlink($imgPath) : null;
			\XF::db()->delete('xa_my_mms', 'mms_id = ?',[$mms_id]);
		}
		if ($page !== 1) {
			$page_params = $cat_id . "&page=" .$page;
		}else{
			$page_params = $cat_id;
		}

		return $this->redirect($this->buildLink('mymmsmsgs/view-by-cat&cat_id=' . $page_params) , \XF::phrase('mms_all_selected_images_has_been_removed_successfully'));
	}


	protected function assertMmsExists($id, $with = null, $phraseKey = null)
	{
		return $this->assertRecordExists('xenarabia\MYMMSMSG\XF:MmsMsgs', $id, $with, $phraseKey);
	}

	protected function checkIfImageInDb($image_name)
	{
		$mms = $this->finder('xenarabia\MYMMSMSG\XF:MmsMsgs')->where('mms_img_name', $image_name)->total();
		if ($mms > 0) {
			return false;
		}

		return true;
	}
}